package com.ftdi;

import com.ftdi.io.JSSCSerialPort;
import com.ftdi.io.SerialPort;
import com.ftdi.util.AppProperties;

public class FT90xSPI {
	
	private final SerialPort sp = new JSSCSerialPort();
	private final int SERIAL_BAUD;
	private final SerialPort.DATABITS SERIAL_DATA;
	private final SerialPort.PARITY SERIAL_PARITY;
	private final SerialPort.STOPBITS SERIAL_STOP;
	
	private String port;
	private boolean isOpen = false;
	
	public FT90xSPI()
	{
		SERIAL_BAUD = Integer.parseInt(AppProperties.getProperties().getProperty("ftdi.ft90x.serial.baud", "19200"));
		
		switch (Integer.parseInt(AppProperties.getProperties().getProperty("ftdi.ft90x.serial.databits", "8")))
		{
		default:
		case 8:
			SERIAL_DATA = SerialPort.DATABITS.DATABITS_8; break;
		case 7:
			SERIAL_DATA = SerialPort.DATABITS.DATABITS_7; break;
		case 6:
			SERIAL_DATA = SerialPort.DATABITS.DATABITS_6; break;
		case 5:
			SERIAL_DATA = SerialPort.DATABITS.DATABITS_5; break;
		}
		
		String parity = AppProperties.getProperties().getProperty("ftdi.ft90x.serial.parity", "N").trim().toLowerCase(); 
		if (parity.equals("n") || parity.equals("none"))
			SERIAL_PARITY = SerialPort.PARITY.PARITY_NONE;
		else if (parity.equals("o") || parity.equals("odd"))
			SERIAL_PARITY = SerialPort.PARITY.PARITY_ODD;
		else if (parity.equals("e") || parity.equals("even"))
			SERIAL_PARITY = SerialPort.PARITY.PARITY_EVEN;
		else if (parity.equals("m") || parity.equals("mark"))
			SERIAL_PARITY = SerialPort.PARITY.PARITY_MARK;
		else if (parity.equals("s") || parity.equals("space"))
			SERIAL_PARITY = SerialPort.PARITY.PARITY_SPACE;
		else
			SERIAL_PARITY = SerialPort.PARITY.PARITY_NONE;
		
		float stop = Float.parseFloat((AppProperties.getProperties().getProperty("ftdi.ft90x.serial.stopbits", "1")));
		
		if (stop == 1.0f)
		{
			SERIAL_STOP = SerialPort.STOPBITS.STOPBITS_1;
		}
		else if (stop == 1.5f)
		{
			SERIAL_STOP = SerialPort.STOPBITS.STOPBITS_1_5;
		}
		else if (stop == 2.0f)
		{
			SERIAL_STOP = SerialPort.STOPBITS.STOPBITS_2;
		}
		else
		{
			SERIAL_STOP = SerialPort.STOPBITS.STOPBITS_1;
		}
	}
	
	/** Open a port to the FT90x SPI bridge
	 * 
	 *  @param port The name of the port to use */
	public void open(String port)
	{
		this.port = port;
		sp.open(port, SERIAL_BAUD, SERIAL_DATA, SERIAL_STOP, SERIAL_PARITY);
		isOpen = true;
	}
	
	/** Close the port to the FT90x SPI bridge */
	public void close()
	{
		if (isOpen)
		{
			sp.close();
			isOpen = false;
			this.port = null;
		}
	}
	
	/** Get the current port of the FT90x SPI Bridge
	 * 
	 *  @return The name of the port or null
	 */
	public String getPort()
	{
		return this.port;
	}
	
	/** Transfer an array of bytes over SPI
	 * 
	 *  @param bytes The bytes to transmit over SPI
	 *  @return An array of the bytes received over SPI
	 */
	public byte[] transfer(byte[] bytes)
	{
		byte rx[];
		
		if (isOpen)
		{
			/* Clear out all the buffers */
			sp.flush(); 
			sp.clear();
			
			sp.write((byte)bytes.length); /* Packet length */
			sp.write(bytes); /* Payload */
			
			/* Wait for it all to transmit */
			sp.flush(); 

			/* Read recevied into a buffer */
			rx = new byte[bytes.length];
			sp.read(rx);
		}
		else
		{
			/* Return nothing */
			rx = new byte[0];
		}
		
		
		return rx;
	}

}
